@@ -22,15 +22,9 @@
 # ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
 # CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 # SOFTWARE.
-from __future__ import absolute_import
-from __future__ import division
-from __future__ import print_function
-from __future__ import unicode_literals
 
-import os
 import sys
 import datetime
-import time
 import copy
 import argparse
 import json
@@ -40,13 +34,12 @@ from functools import wraps
 from decimal import Decimal
 
 from .import util
-from .util import print_msg, format_satoshis, print_stderr
+from .util import bfh, bh2u, format_satoshis
 from .import bitcoin
 from .bitcoin import is_address,  hash_160, COIN, TYPE_ADDRESS
+from .i18n import _
 from .transaction import Transaction
-from .import paymentrequest
 from .paymentrequest import PR_PAID, PR_UNPAID, PR_UNKNOWN, PR_EXPIRED
-from .import contacts
 from .plugins import run_hook
 
 known_commands = {}
@@ -651,19 +644,19 @@ class Commands:
     @command('w')
     def clearrequests(self):
         """Remove all payment requests"""
-        for k in self.wallet.receive_requests.keys():
+        for k in list(self.wallet.receive_requests.keys()):
             self.wallet.remove_payment_request(k, self.config)
 
     @command('n')
     def notify(self, address, URL):
         """Watch an address. Everytime the address changes, a http POST is sent to the URL."""
         def callback(x):
-            import urllib2
+            import urllib.request
             headers = {'content-type':'application/json'}
             data = {'address':address, 'status':x.get('result')}
             try:
-                req = urllib2.Request(URL, json.dumps(data), headers)
-                response_stream = urllib2.urlopen(req, timeout=5)
+                req = urllib.request.Request(URL, json.dumps(data), headers)
+                response_stream = urllib.request.urlopen(req, timeout=5)
                 util.print_error('Got Response for %s' % address)
             except BaseException as e:
                 util.print_error(str(e))
