@@ -24,21 +24,16 @@
 # CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 # SOFTWARE.
 
-import struct
-
 from unicodedata import normalize
 
-from .version import *
 from . import bitcoin
-from .bitcoin import pw_encode, pw_decode, bip32_root, bip32_private_derivation, bip32_public_derivation, bip32_private_key, deserialize_xprv, deserialize_xpub
-from .bitcoin import public_key_from_private_key, public_key_to_p2pkh
 from .bitcoin import *
 
-from .bitcoin import is_old_seed, is_new_seed, is_seed
 from .util import PrintError, InvalidPassword, hfu
 from .mnemonic import Mnemonic, load_wordlist
 from .plugins import run_hook
 
+
 class KeyStore(PrintError):
 
     def has_seed(self):
@@ -678,7 +673,7 @@ is_bip32_key = lambda x: is_xprv(x) or is_xpub(x)
 
 def bip44_derivation(account_id, segwit=False):
     bip  = 49 if segwit else 44
-    coin = 1 if bitcoin.TESTNET else 0
+    coin = 1 if bitcoin.NetworkConstants.TESTNET else 0
     return "m/%d'/%d'/%d'" % (bip, coin, int(account_id))
 
 def from_seed(seed, passphrase, is_p2sh):
