@@ -1,4 +1,4 @@
-#!/usr/bin/env python2
+#!/usr/bin/env python
 #
 # Electrum - lightweight Bitcoin client
 # Copyright (C) 2015 Thomas Voegtlin
@@ -33,15 +33,11 @@
 
 """Pure-Python RSA implementation."""
 
-
-from __future__ import print_function
 import os
 import math
-import base64
-import binascii
 import hashlib
 
-from pem import *
+from .pem import *
 
 
 def SHA1(x):
