@@ -1,5 +1,3 @@
-#!/usr/bin/env python2
-#
 # Electrum - Lightweight Bitcoin Client
 # Copyright (c) 2012 Thomas Voegtlin
 #
@@ -22,10 +20,8 @@
 # ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
 # CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 # SOFTWARE.
-
-
-from util import ThreadJob
-from bitcoin import *
+from .util import ThreadJob
+from .bitcoin import *
 
 
 class SPV(ThreadJob):
@@ -44,7 +40,7 @@ class SPV(ThreadJob):
         unverified = self.wallet.get_unverified_txs()
         for tx_hash, tx_height in unverified.items():
             # do not request merkle branch before headers are available
-            if tx_height>0 and tx_hash not in self.merkle_roots and tx_height <= lh:
+            if (tx_height > 0) and (tx_hash not in self.merkle_roots) and (tx_height <= lh):
                 request = ('blockchain.transaction.get_merkle',
                            [tx_hash, tx_height])
                 self.network.send([request], self.verify_merkle)
@@ -82,7 +78,7 @@ class SPV(ThreadJob):
         h = hash_decode(target_hash)
         for i in range(len(merkle_s)):
             item = merkle_s[i]
-            h = Hash( hash_decode(item) + h ) if ((pos >> i) & 1) else Hash( h + hash_decode(item) )
+            h = Hash(hash_decode(item) + h) if ((pos >> i) & 1) else Hash(h + hash_decode(item))
         return hash_encode(h)
 
     def undo_verifications(self):
