@@ -28,18 +28,14 @@
 
 
 import os
-import hashlib
-import ast
 import threading
 import random
 import time
 import json
 import copy
-import re
-import stat
 import errno
 from functools import partial
-from collections import namedtuple, defaultdict
+from collections import defaultdict
 
 from .i18n import _
 from .util import NotEnoughFunds, PrintError, UserCancelled, profiler, format_satoshis
@@ -56,15 +52,12 @@ from . import bitcoin
 from . import coinchooser
 from .synchronizer import Synchronizer
 from .verifier import SPV
-from .mnemonic import Mnemonic
 
 from . import paymentrequest
 from .paymentrequest import PR_PAID, PR_UNPAID, PR_UNKNOWN, PR_EXPIRED
 from .paymentrequest import InvoiceStore
 from .contacts import Contacts
 
-from .storage import WalletStorage
-
 TX_STATUS = [
     _('Replaceable'),
     _('Unconfirmed parent'),
@@ -893,15 +886,28 @@ class Abstract_Wallet(PrintError):
             inputs.append(item)
 
     def sweep(self, privkeys, network, config, recipient, fee=None, imax=100):
+
+        def find_utxos_for_privkey(txin_type, privkey, compressed):
+            pubkey = bitcoin.public_key_from_private_key(privkey, compressed)
+            self._append_utxos_to_inputs(inputs, network, pubkey, txin_type, imax)
+            keypairs[pubkey] = privkey, compressed
+
         inputs = []
         keypairs = {}
         for sec in privkeys:
             txin_type, privkey, compressed = bitcoin.deserialize_privkey(sec)
-            pubkey = bitcoin.public_key_from_private_key(privkey, compressed)
-            self._append_utxos_to_inputs(inputs, network, pubkey, txin_type, imax)
-            if txin_type == 'p2pkh':  # WIF serialization is ambiguous :(
-                self._append_utxos_to_inputs(inputs, network, pubkey, 'p2pk', imax)
-            keypairs[pubkey] = privkey, compressed
+
+            find_utxos_for_privkey(txin_type, privkey, compressed)
+
+            # do other lookups to increase support coverage
+            if is_minikey(sec):
+                # minikeys don't have a compressed byte
+                # we lookup both compressed and uncompressed pubkeys
+                find_utxos_for_privkey(txin_type, privkey, not compressed)
+            elif txin_type == 'p2pkh':
+                # WIF serialization does not distinguish p2pkh and p2pk
+                # we also search for pay-to-pubkey outputs
+                find_utxos_for_privkey('p2pk', privkey, compressed)
 
         if not inputs:
             raise BaseException(_('No inputs found. (Note that inputs need to be confirmed)'))
@@ -1268,7 +1274,6 @@ class Abstract_Wallet(PrintError):
         self.storage.put('payment_requests', self.receive_requests)
 
     def add_payment_request(self, req, config):
-        import os
         addr = req['address']
         amount = req.get('amount')
         message = req.get('memo')
@@ -1287,7 +1292,7 @@ class Abstract_Wallet(PrintError):
                 except OSError as exc:
                     if exc.errno != errno.EEXIST:
                         raise
-            with open(os.path.join(path, key), 'w') as f:
+            with open(os.path.join(path, key), 'wb') as f:
                 f.write(pr.SerializeToString())
             # reload
             req = self.get_payment_request(addr, config)
@@ -1339,6 +1344,9 @@ class Abstract_Wallet(PrintError):
     def has_password(self):
         return self.storage.get('use_encryption', False)
 
+    def check_password(self, password):
+        self.keystore.check_password(password)
+
     def sign_message(self, address, message, password):
         index = self.get_address_index(address)
         return self.keystore.sign_message(index, message, password)
@@ -1364,9 +1372,6 @@ class Simple_Wallet(Abstract_Wallet):
     def can_change_password(self):
         return self.keystore.can_change_password()
 
-    def check_password(self, password):
-        self.keystore.check_password(password)
-
     def update_password(self, old_pw, new_pw, encrypt=False):
         if old_pw is None and self.has_password():
             raise InvalidPassword()
@@ -1394,9 +1399,6 @@ class Imported_Wallet(Simple_Wallet):
     def get_keystores(self):
         return [self.keystore] if self.keystore else []
 
-    def check_password(self, password):
-        self.keystore.check_password(password)
-
     def can_import_privkey(self):
         return bool(self.keystore)
 
@@ -1792,9 +1794,6 @@ class Multisig_Wallet(Deterministic_Wallet):
         self.storage.set_password(new_pw, encrypt)
         self.storage.write()
 
-    def check_password(self, password):
-        self.keystore.check_password(password)
-
     def has_seed(self):
         return self.keystore.has_seed()
 
