@@ -1,16 +1,15 @@
 import os.path
 import time
-import traceback
 import sys
-import threading
 import platform
-import Queue
+import queue
 from collections import namedtuple
 from functools import partial
 
 from electrum.i18n import _
-from PyQt4.QtGui import *
-from PyQt4.QtCore import *
+from PyQt5.QtGui import *
+from PyQt5.QtCore import *
+from PyQt5.QtWidgets import *
 
 if platform.system() == 'Windows':
     MONOSPACE_FONT = 'Lucida Console'
@@ -19,15 +18,10 @@ elif platform.system() == 'Darwin':
 else:
     MONOSPACE_FONT = 'monospace'
 
-GREEN_BG = "QWidget {background-color:#80ff80;}"
-RED_BG = "QWidget {background-color:#ffcccc;}"
-RED_FG = "QWidget {color:red;}"
-BLUE_FG = "QWidget {color:blue;}"
-BLACK_FG = "QWidget {color:black;}"
 
 dialogs = []
 
-from electrum.paymentrequest import PR_UNPAID, PR_PAID, PR_UNKNOWN, PR_EXPIRED
+from electrum.paymentrequest import PR_UNPAID, PR_PAID, PR_EXPIRED
 
 pr_icons = {
     PR_UNPAID:":icons/unpaid.png",
@@ -51,10 +45,11 @@ expiration_values = [
 
 class Timer(QThread):
     stopped = False
+    timer_signal = pyqtSignal()
 
     def run(self):
         while not self.stopped:
-            self.emit(SIGNAL('timersignal'))
+            self.timer_signal.emit()
             time.sleep(0.5)
 
     def stop(self):
@@ -105,7 +100,7 @@ class HelpLabel(QLabel):
         self.font = QFont()
 
     def mouseReleaseEvent(self, x):
-        QMessageBox.information(self, 'Help', self.help_text, 'OK')
+        QMessageBox.information(self, 'Help', self.help_text)
 
     def enterEvent(self, event):
         self.font.setUnderline(True)
@@ -129,7 +124,7 @@ class HelpButton(QPushButton):
         self.clicked.connect(self.onclick)
 
     def onclick(self):
-        QMessageBox.information(self, 'Help', self.help_text, 'OK')
+        QMessageBox.information(self, 'Help', self.help_text)
 
 class Buttons(QHBoxLayout):
     def __init__(self, *buttons):
@@ -205,7 +200,7 @@ class MessageBoxMixin(object):
     def msg_box(self, icon, parent, title, text, buttons=QMessageBox.Ok,
                 defaultButton=QMessageBox.NoButton):
         parent = parent or self.top_level_window()
-        d = QMessageBox(icon, title, text, buttons, parent)
+        d = QMessageBox(icon, title, str(text), buttons, parent)
         d.setWindowModality(Qt.WindowModal)
         d.setDefaultButton(defaultButton)
         return d.exec_()
@@ -254,10 +249,10 @@ def line_dialog(parent, title, label, ok_label, default=None):
     l.addWidget(txt)
     l.addLayout(Buttons(CancelButton(dialog), OkButton(dialog, ok_label)))
     if dialog.exec_():
-        return unicode(txt.text())
+        return txt.text()
 
 def text_dialog(parent, title, label, ok_label, default=None):
-    from qrtextedit import ScanQRTextEdit
+    from .qrtextedit import ScanQRTextEdit
     dialog = WindowModalDialog(parent, title)
     dialog.setMinimumWidth(500)
     l = QVBoxLayout()
@@ -269,7 +264,7 @@ def text_dialog(parent, title, label, ok_label, default=None):
     l.addWidget(txt)
     l.addLayout(Buttons(CancelButton(dialog), OkButton(dialog, ok_label)))
     if dialog.exec_():
-        return unicode(txt.toPlainText())
+        return txt.toPlainText()
 
 class ChoicesLayout(object):
     def __init__(self, msg, choices, on_clicked=None, checked_index=0):
@@ -335,15 +330,15 @@ def filename_field(parent, config, defaultname, select_msg):
 
     hbox = QHBoxLayout()
 
-    directory = config.get('io_dir', unicode(os.path.expanduser('~')))
+    directory = config.get('io_dir', os.path.expanduser('~'))
     path = os.path.join( directory, defaultname )
     filename_e = QLineEdit()
     filename_e.setText(path)
 
     def func():
-        text = unicode(filename_e.text())
+        text = filename_e.text()
         _filter = "*.csv" if text.endswith(".csv") else "*.json" if text.endswith(".json") else None
-        p = unicode( QFileDialog.getSaveFileName(None, select_msg, text, _filter))
+        p, __ = QFileDialog.getSaveFileName(None, select_msg, text, _filter)
         if p:
             filename_e.setText(p)
 
@@ -354,7 +349,7 @@ def filename_field(parent, config, defaultname, select_msg):
     vbox.addLayout(hbox)
 
     def set_csv(v):
-        text = unicode(filename_e.text())
+        text = filename_e.text()
         text = text.replace(".json",".csv") if v else text.replace(".csv",".json")
         filename_e.setText(text)
 
@@ -399,11 +394,11 @@ class MyTreeWidget(QTreeWidget):
         self.header().setStretchLastSection(False)
         for col in range(len(headers)):
             sm = QHeaderView.Stretch if col == self.stretch_column else QHeaderView.ResizeToContents
-            self.header().setResizeMode(col, sm)
+            self.header().setSectionResizeMode(col, sm)
 
     def editItem(self, item, column):
         if column in self.editable_columns:
-            self.editing_itemcol = (item, column, unicode(item.text(column)))
+            self.editing_itemcol = (item, column, item.text(column))
             # Calling setFlags causes on_changed events for some reason
             item.setFlags(item.flags() | Qt.ItemIsEditable)
             QTreeWidget.editItem(self, item, column)
@@ -430,13 +425,12 @@ class MyTreeWidget(QTreeWidget):
         # on 'enter' we show the menu
         pt = self.visualItemRect(item).bottomLeft()
         pt.setX(50)
-        self.emit(SIGNAL('customContextMenuRequested(const QPoint&)'), pt)
+        self.customContextMenuRequested.emit(pt)
 
     def createEditor(self, parent, option, index):
         self.editor = QStyledItemDelegate.createEditor(self.itemDelegate(),
                                                        parent, option, index)
-        self.editor.connect(self.editor, SIGNAL("editingFinished()"),
-                            self.editing_finished)
+        self.editor.editingFinished.connect(self.editing_finished)
         return self.editor
 
     def editing_finished(self):
@@ -464,8 +458,8 @@ class MyTreeWidget(QTreeWidget):
 
     def on_edited(self, item, column, prior):
         '''Called only when the text actually changes'''
-        key = str(item.data(0, Qt.UserRole).toString())
-        text = unicode(item.text(column))
+        key = item.data(0, Qt.UserRole)
+        text = item.text(column)
         self.parent.wallet.set_label(key, text)
         self.parent.history_list.update_labels()
         self.parent.update_completions()
@@ -495,10 +489,10 @@ class MyTreeWidget(QTreeWidget):
 
     def filter(self, p):
         columns = self.__class__.filter_columns
-        p = unicode(p).lower()
+        p = p.lower()
         self.current_filter = p
         for item in self.get_leaves(self.invisibleRootItem()):
-            item.setHidden(all([unicode(item.text(column)).lower().find(p) == -1
+            item.setHidden(all([item.text(column).lower().find(p) == -1
                                 for column in columns]))
 
 
@@ -529,8 +523,11 @@ class ButtonsWidget(QWidget):
 
     def addCopyButton(self, app):
         self.app = app
-        f = lambda: self.app.clipboard().setText(str(self.text()))
-        self.addButton(":icons/copy.png", f, _("Copy to clipboard"))
+        self.addButton(":icons/copy.png", self.on_copy, _("Copy to clipboard"))
+
+    def on_copy(self):
+        self.app.clipboard().setText(self.text())
+        QToolTip.showText(QCursor.pos(), _("Text copied to clipboard"), self)
 
 class ButtonsLineEdit(QLineEdit, ButtonsWidget):
     def __init__(self, text=None):
@@ -565,7 +562,7 @@ class TaskThread(QThread):
     def __init__(self, parent, on_error=None):
         super(TaskThread, self).__init__(parent)
         self.on_error = on_error
-        self.tasks = Queue.Queue()
+        self.tasks = queue.Queue()
         self.doneSig.connect(self.on_done)
         self.start()
 
@@ -595,8 +592,43 @@ class TaskThread(QThread):
         self.tasks.put(None)
 
 
+class ColorSchemeItem:
+    def __init__(self, fg_color, bg_color):
+        self.colors = (fg_color, bg_color)
+
+    def _get_color(self, background):
+        return self.colors[(int(background) + int(ColorScheme.dark_scheme)) % 2]
+
+    def as_stylesheet(self, background=False):
+        css_prefix = "background-" if background else ""
+        color = self._get_color(background)
+        return "QWidget {{ {}color:{}; }}".format(css_prefix, color)
+
+    def as_color(self, background=False):
+        color = self._get_color(background)
+        return QColor(color)
+
+
+class ColorScheme:
+    dark_scheme = False
+
+    GREEN = ColorSchemeItem("#117c11", "#8af296")
+    RED = ColorSchemeItem("#7c1111", "#f18c8c")
+    BLUE = ColorSchemeItem("#123b7c", "#8cb3f2")
+    DEFAULT = ColorSchemeItem("black", "white")
+
+    @staticmethod
+    def has_dark_background(widget):
+        brightness = sum(widget.palette().color(QPalette.Background).getRgb()[0:3])
+        return brightness < (255*3/2)
+
+    @staticmethod
+    def update_from_widget(widget):
+        if ColorScheme.has_dark_background(widget):
+            ColorScheme.dark_scheme = True
+
 if __name__ == "__main__":
     app = QApplication([])
-    t = WaitingDialog(None, 'testing ...', lambda: [time.sleep(1)], lambda x: QMessageBox.information(None, 'done', "done", _('OK')))
+    t = WaitingDialog(None, 'testing ...', lambda: [time.sleep(1)], lambda x: QMessageBox.information(None, 'done', "done"))
     t.start()
     app.exec_()
