@@ -1,4 +1,4 @@
-#!/usr/bin/env python2
+#!/usr/bin/env python
 #
 # Electrum - lightweight Bitcoin client
 # Copyright (C) 2015 Thomas Voegtlin
@@ -22,10 +22,8 @@
 # ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
 # CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 # SOFTWARE.
-
-from util import *
+from .util import *
 from electrum.i18n import _
-from electrum.bitcoin import is_address
 
 
 class UTXOList(MyTreeWidget):
@@ -54,11 +52,11 @@ class UTXOList(MyTreeWidget):
             utxo_item.setFont(4, QFont(MONOSPACE_FONT))
             utxo_item.setData(0, Qt.UserRole, name)
             if self.wallet.is_frozen(address):
-                utxo_item.setBackgroundColor(0, QColor('lightblue'))
+                utxo_item.setBackground(0, ColorScheme.BLUE.as_color(True))
             self.addChild(utxo_item)
 
     def create_menu(self, position):
-        selected = [str(x.data(0, Qt.UserRole).toString()) for x in self.selectedItems()]
+        selected = [x.data(0, Qt.UserRole) for x in self.selectedItems()]
         if not selected:
             return
         menu = QMenu()
