@@ -1,11 +1,10 @@
 from decimal import Decimal
 _ = lambda x:x
 #from i18n import _
-from electrum.wallet import WalletStorage, Wallet
-from electrum.util import format_satoshis, set_verbosity, StoreDict
-from electrum.bitcoin import is_valid, COIN, TYPE_ADDRESS
-from electrum.network import filter_protocol
-import sys, getpass, datetime
+from electrum import WalletStorage, Wallet
+from electrum.util import format_satoshis, set_verbosity
+from electrum.bitcoin import is_address, COIN, TYPE_ADDRESS
+import getpass, datetime
 
 # minimal fdisk like gui for console usage
 # written by rofl0r, with some bits stolen from the text gui (ncurses)
@@ -17,8 +16,11 @@ class ElectrumGui:
         self.network = daemon.network
         storage = WalletStorage(config.get_wallet_path())
         if not storage.file_exists:
-            print "Wallet not found. try 'electrum create'"
+            print("Wallet not found. try 'electrum create'")
             exit()
+        if storage.is_encrypted():
+            password = getpass.getpass('Password:', stream=None)
+            storage.decrypt(password)
 
         self.done = 0
         self.last_balance = ""
@@ -32,7 +34,7 @@ class ElectrumGui:
 
         self.wallet = Wallet(storage)
         self.wallet.start_threads(self.network)
-        self.contacts = StoreDict(self.config, 'contacts')
+        self.contacts = self.wallet.contacts
 
         self.network.register_callback(self.on_network, ['updated', 'banner'])
         self.commands = [_("[h] - displays this help text"), \
@@ -54,8 +56,8 @@ class ElectrumGui:
 
     def main_command(self):
         self.print_balance()
-        c = raw_input("enter command: ")
-        if   c == "h" : self.print_commands()
+        c = input("enter command: ")
+        if c == "h" : self.print_commands()
         elif c == "i" : self.print_history()
         elif c == "o" : self.enter_order()
         elif c == "p" : self.print_order()
@@ -135,10 +137,10 @@ class ElectrumGui:
               + "\nfee: " + self.str_fee + ", desc: " + self.str_description)
 
     def enter_order(self):
-        self.str_recipient = raw_input("Pay to: ")
-        self.str_description = raw_input("Description : ")
-        self.str_amount = raw_input("Amount: ")
-        self.str_fee = raw_input("Fee: ")
+        self.str_recipient = input("Pay to: ")
+        self.str_description = input("Description : ")
+        self.str_amount = input("Amount: ")
+        self.str_fee = input("Fee: ")
 
     def send_order(self):
         self.do_send()
@@ -160,7 +162,7 @@ class ElectrumGui:
         while self.done == 0: self.main_command()
 
     def do_send(self):
-        if not is_valid(self.str_recipient):
+        if not is_address(self.str_recipient):
             print(_('Invalid Bitcoin address'))
             return
         try:
@@ -183,7 +185,7 @@ class ElectrumGui:
 
         c = ""
         while c != "y":
-            c = raw_input("ok to send (y/n)?")
+            c = input("ok to send (y/n)?")
             if c == "n": return
 
         try:
