@@ -5,7 +5,8 @@
 
 try:
     import electrum
-    from electrum.bitcoin import TYPE_ADDRESS, var_int, msg_magic, Hash, verify_message, public_key_to_p2pkh, EncodeAES, DecodeAES
+    from electrum.bitcoin import TYPE_ADDRESS, push_script, var_int, msg_magic, Hash, verify_message, pubkey_from_signature, point_to_ser, public_key_to_p2pkh, EncodeAES, DecodeAES, MyVerifyingKey
+    from electrum.transaction import Transaction
     from electrum.i18n import _
     from electrum.keystore import Hardware_KeyStore
     from ..hw_wallet import HW_PluginBase
@@ -15,9 +16,16 @@ try:
     import hid
     import json
     import math
+    import binascii
+    import struct
     import hashlib
+    import requests
+    import base64
+    import os
+    import sys
     from ecdsa.ecdsa import generator_secp256k1
     from ecdsa.util import sigencode_der
+    from ecdsa.curves import SECP256k1
     DIGIBOX = True
 except ImportError as e:
     DIGIBOX = False
@@ -28,15 +36,19 @@ except ImportError as e:
 # USB HID interface
 #
 
+def to_hexstr(s):
+    return binascii.hexlify(s).decode('ascii')
+
 class DigitalBitbox_Client():
 
-    def __init__(self, hidDevice):
+    def __init__(self, plugin, hidDevice):
+        self.plugin = plugin
         self.dbb_hid = hidDevice
         self.opened = True
         self.password = None
         self.isInitialized = False
         self.setupRunning = False
-        self.hidBufSize = 4096
+        self.usbReportSize = 64 # firmware > v2.0.0
 
 
     def close(self):
@@ -67,17 +79,22 @@ class DigitalBitbox_Client():
     def is_paired(self):
         return self.password is not None
 
-
-    def get_xpub(self, bip32_path):
+    def _get_xpub(self, bip32_path):
         if self.check_device_dialog():
-            msg = '{"xpub":"' + bip32_path + '"}' 
-            reply = self.hid_send_encrypt(msg)
+            return self.hid_send_encrypt(b'{"xpub": "%s"}' % bip32_path.encode('utf8'))
+
+
+    def get_xpub(self, bip32_path, xtype):
+        assert xtype == 'standard'
+        reply = self._get_xpub(bip32_path)
+        if reply:
             return reply['xpub']
-        return None
+        else:
+            raise BaseException('no reply')
 
 
     def dbb_has_password(self):
-        reply = self.hid_send_plain('{"ping":""}')
+        reply = self.hid_send_plain(b'{"ping":""}')
         if 'ping' not in reply:
             raise Exception('Device communication error. Please unplug and replug your Digital Bitbox.')
         if reply['ping'] == 'password':
@@ -87,7 +104,7 @@ class DigitalBitbox_Client():
 
     def stretch_key(self, key):
         import pbkdf2, hmac
-        return pbkdf2.PBKDF2(key, 'Digital Bitbox', iterations = 20480, macmodule = hmac, digestmodule = hashlib.sha512).read(64).encode('hex')
+        return binascii.hexlify(pbkdf2.PBKDF2(key, b'Digital Bitbox', iterations = 20480, macmodule = hmac, digestmodule = hashlib.sha512).read(64))
 
 
     def backup_password_dialog(self):
@@ -101,7 +118,7 @@ class DigitalBitbox_Client():
             elif len(password) > 64:
                 msg = _("Password must have less than 64 characters.\r\n\r\nEnter password:")
             else:
-                return str(password)
+                return password.encode('utf8')
 
 
     def password_dialog(self, msg):
@@ -114,7 +131,7 @@ class DigitalBitbox_Client():
             elif len(password) > 64:
                 msg = _("Password must have less than 64 characters.\r\n\r\nEnter password:")
             else:
-                self.password = str(password)
+                self.password = password.encode('utf8')
                 return True
 
 
@@ -128,16 +145,16 @@ class DigitalBitbox_Client():
                     "You cannot access your coins or a backup without the password.\r\n" \
                     "A backup is saved automatically when generating a new wallet.")
             if self.password_dialog(msg):
-                reply = self.hid_send_plain('{"password":"' + self.password + '"}')
+                reply = self.hid_send_plain(b'{"password":"' + self.password + b'"}')
             else:
                 return False
-        
+
         # Get password from user if not yet set
         msg = _("Enter your Digital Bitbox password:")
         while self.password is None:
             if not self.password_dialog(msg):
                 return False
-            reply = self.hid_send_encrypt('{"led":"blink"}')
+            reply = self.hid_send_encrypt(b'{"led":"blink"}')
             if 'error' in reply:
                 self.password = None
                 if reply['error']['code'] == 109:
@@ -149,17 +166,17 @@ class DigitalBitbox_Client():
                     msg = _("Unexpected error occurred.\r\n\r\n"  \
                             + reply['error']['message'] + "\r\n\r\n" \
                             "Enter your Digital Bitbox password:")
-        
+
         # Initialize device if not yet initialized
         if not self.setupRunning:
             self.isInitialized = True # Wallet exists. Electrum code later checks if the device matches the wallet
         elif not self.isInitialized:
-            reply = self.hid_send_encrypt('{"device":"info"}')
-            if reply['device']['id'] <> "":
+            reply = self.hid_send_encrypt(b'{"device":"info"}')
+            if reply['device']['id'] != "":
                 self.recover_or_erase_dialog() # Already seeded
             else:
                 self.seed_device_dialog() # Seed if not initialized
-        
+            self.mobile_pairing_dialog()
         return self.isInitialized
 
 
@@ -180,7 +197,9 @@ class DigitalBitbox_Client():
             if not self.dbb_load_backup():
                 return
         else:
-            pass # Use existing seed
+            if self.hid_send_encrypt(b'{"device":"info"}')['device']['lock']:
+                raise Exception("Full 2FA enabled. This is not supported yet.")
+            # Use existing seed
         self.isInitialized = True
 
 
@@ -201,11 +220,50 @@ class DigitalBitbox_Client():
                 return
         self.isInitialized = True
 
+    def mobile_pairing_dialog(self):
+        dbb_user_dir = None
+        if sys.platform == 'darwin':
+            dbb_user_dir = os.path.join(os.environ.get("HOME", ""), "Library", "Application Support", "DBB")
+        elif sys.platform == 'win32':
+            dbb_user_dir = os.path.join(os.environ["APPDATA"], "DBB")
+        else:
+            dbb_user_dir = os.path.join(os.environ["HOME"], ".dbb")
+
+        if not dbb_user_dir:
+            return
+
+        try:
+            with open(os.path.join(dbb_user_dir, "config.dat")) as f:
+                dbb_config = json.load(f)
+        except (FileNotFoundError, json.JSONDecodeError):
+            return
+
+        if 'encryptionprivkey' not in dbb_config or 'comserverchannelid' not in dbb_config:
+            return
+
+        choices = [
+            _('Do not pair'),
+            _('Import pairing from the digital bitbox desktop app'),
+        ]
+        try:
+            reply = self.handler.win.query_choice(_('Mobile pairing options'), choices)
+        except Exception:
+            return # Back button pushed
+
+        if reply == 0:
+            if self.plugin.is_mobile_paired():
+                del self.plugin.digitalbitbox_config['encryptionprivkey']
+                del self.plugin.digitalbitbox_config['comserverchannelid']
+        elif reply == 1:
+            # import pairing from dbb app
+            self.plugin.digitalbitbox_config['encryptionprivkey'] = dbb_config['encryptionprivkey']
+            self.plugin.digitalbitbox_config['comserverchannelid'] = dbb_config['comserverchannelid']
+        self.plugin.config.set_key('digitalbitbox', self.plugin.digitalbitbox_config)
 
     def dbb_generate_wallet(self):
         key = self.stretch_key(self.password)
-        filename = "Electrum-" + time.strftime("%Y-%m-%d-%H-%M-%S") + ".pdf"
-        msg = '{"seed":{"source": "create", "key": "%s", "filename": "%s", "entropy": "%s"}}' % (key, filename, 'Digital Bitbox Electrum Plugin')
+        filename = ("Electrum-" + time.strftime("%Y-%m-%d-%H-%M-%S") + ".pdf").encode('utf8')
+        msg = b'{"seed":{"source": "create", "key": "%s", "filename": "%s", "entropy": "%s"}}' % (key, filename, b'Digital Bitbox Electrum Plugin')
         reply = self.hid_send_encrypt(msg)
         if 'error' in reply:
             raise Exception(reply['error']['message'])
@@ -215,7 +273,7 @@ class DigitalBitbox_Client():
         self.handler.show_message(_("Are you sure you want to erase the Digital Bitbox?\r\n\r\n" \
                                     "To continue, touch the Digital Bitbox's light for 3 seconds.\r\n\r\n" \
                                     "To cancel, briefly touch the light or wait for the timeout."))
-        hid_reply = self.hid_send_encrypt('{"reset":"__ERASE__"}')
+        hid_reply = self.hid_send_encrypt(b'{"reset":"__ERASE__"}')
         self.handler.clear_dialog()
         if 'error' in hid_reply:
             raise Exception(hid_reply['error']['message'])
@@ -225,7 +283,7 @@ class DigitalBitbox_Client():
 
 
     def dbb_load_backup(self, show_msg=True):
-        backups = self.hid_send_encrypt('{"backup":"list"}')
+        backups = self.hid_send_encrypt(b'{"backup":"list"}')
         if 'error' in backups:
             raise Exception(backups['error']['message'])
         try:
@@ -240,7 +298,7 @@ class DigitalBitbox_Client():
             self.handler.show_message(_("Loading backup...\r\n\r\n" \
                                         "To continue, touch the Digital Bitbox's light for 3 seconds.\r\n\r\n" \
                                         "To cancel, briefly touch the light or wait for the timeout."))
-        msg = '{"seed":{"source": "backup", "key": "%s", "filename": "%s"}}' % (key, backups['backup'][f])
+        msg = b'{"seed":{"source": "backup", "key": "%s", "filename": "%s"}}' % (key, backups['backup'][f].encode('utf8'))
         hid_reply = self.hid_send_encrypt(msg)
         self.handler.clear_dialog()
         if 'error' in hid_reply:
@@ -248,15 +306,57 @@ class DigitalBitbox_Client():
         return True
 
 
+    def hid_send_frame(self, data):
+        HWW_CID = 0xFF000000
+        HWW_CMD = 0x80 + 0x40 + 0x01
+        data_len = len(data)
+        seq = 0;
+        idx = 0;
+        write = []
+        while idx < data_len:
+            if idx == 0:
+                # INIT frame
+                write = data[idx : idx + min(data_len, self.usbReportSize - 7)]
+                self.dbb_hid.write(b'\0' + struct.pack(">IBH", HWW_CID, HWW_CMD, data_len & 0xFFFF) + write + b'\xEE' * (self.usbReportSize - 7 - len(write)))
+            else:
+                # CONT frame
+                write = data[idx : idx + min(data_len, self.usbReportSize - 5)]
+                self.dbb_hid.write(b'\0' + struct.pack(">IB", HWW_CID, seq) + write + b'\xEE' * (self.usbReportSize - 5 - len(write)))
+                seq += 1
+            idx += len(write)
+
+
+    def hid_read_frame(self):
+        # INIT response
+        read = bytearray(self.dbb_hid.read(self.usbReportSize))
+        cid = ((read[0] * 256 + read[1]) * 256 + read[2]) * 256 + read[3]
+        cmd = read[4]
+        data_len = read[5] * 256 + read[6]
+        data = read[7:]
+        idx = len(read) - 7;
+        while idx < data_len:
+            # CONT response
+            read = bytearray(self.dbb_hid.read(self.usbReportSize))
+            data += read[5:]
+            idx += len(read) - 5
+        return data
+
+
     def hid_send_plain(self, msg):
         reply = ""
         try:
-            self.dbb_hid.write('\0' + bytearray(msg) + '\0' * (self.hidBufSize - len(msg)))
-            r = []
-            while len(r) < self.hidBufSize:
-                r = r + self.dbb_hid.read(self.hidBufSize)
-            r = str(bytearray(r)).rstrip(' \t\r\n\0')
-            r = r.replace("\0", '')
+            serial_number = self.dbb_hid.get_serial_number_string()
+            if "v2.0." in serial_number or "v1." in serial_number:
+                hidBufSize = 4096
+                self.dbb_hid.write('\0' + msg + '\0' * (hidBufSize - len(msg)))
+                r = bytearray()
+                while len(r) < hidBufSize:
+                    r += bytearray(self.dbb_hid.read(hidBufSize))
+            else:
+                self.hid_send_frame(msg)
+                r = self.hid_read_frame()
+            r = r.rstrip(b' \t\r\n\0')
+            r = r.replace(b"\0", b'')
             reply = json.loads(r)
         except Exception as e:
             print_error('Exception caught ' + str(e))
@@ -281,7 +381,7 @@ class DigitalBitbox_Client():
 
 
 # ----------------------------------------------------------------------------------
-# 
+#
 #
 
 class DigitalBitbox_KeyStore(Hardware_KeyStore):
@@ -312,19 +412,21 @@ class DigitalBitbox_KeyStore(Hardware_KeyStore):
     def sign_message(self, sequence, message, password):
         sig = None
         try:
+            message = message.encode('utf8')
             inputPath = self.get_derivation() + "/%d/%d" % sequence
-            inputHash = Hash(msg_magic(message)).encode('hex')
+            msg_hash = Hash(msg_magic(message))
+            inputHash = to_hexstr(msg_hash)
             hasharray = []
             hasharray.append({'hash': inputHash, 'keypath': inputPath})
             hasharray = json.dumps(hasharray)
-            
-            msg = '{"sign":{"meta":"sign message", "data":%s}}' % (hasharray)
-            
+
+            msg = b'{"sign":{"meta":"sign message", "data":%s}}' % hasharray.encode('utf8')
+
             dbb_client = self.plugin.get_client(self)
-            
+
             if not dbb_client.is_paired():
                 raise Exception("Could not sign message.")
-            
+
             reply = dbb_client.hid_send_encrypt(msg)
             self.handler.show_message(_("Signing message ...\r\n\r\n" \
                                         "To continue, touch the Digital Bitbox's blinking light for 3 seconds.\r\n\r\n" \
@@ -334,21 +436,32 @@ class DigitalBitbox_KeyStore(Hardware_KeyStore):
 
             if 'error' in reply:
                 raise Exception(reply['error']['message'])
-          
+
             if 'sign' not in reply:
                 raise Exception("Could not sign message.")
 
-            for i in range(4):
-                sig = chr(27 + i + 4) + reply['sign'][0]['sig'].decode('hex')
-                try:
-                    addr = public_key_to_p2pkh(reply['sign'][0]['pubkey'].decode('hex'))
-                    if verify_message(addr, sig, message):
-                        break
-                except Exception:
-                    continue
-            else:
-                raise Exception("Could not sign message")
-            
+            if 'recid' in reply['sign'][0]:
+                # firmware > v2.1.1
+                sig = bytes([27 + int(reply['sign'][0]['recid'], 16) + 4]) + binascii.unhexlify(reply['sign'][0]['sig'])
+                pk, compressed = pubkey_from_signature(sig, msg_hash)
+                pk = point_to_ser(pk.pubkey.point, compressed)
+                addr = public_key_to_p2pkh(pk)
+                if verify_message(addr, sig, message) is False:
+                    raise Exception("Could not sign message")
+            elif 'pubkey' in reply['sign'][0]:
+                # firmware <= v2.1.1
+                for i in range(4):
+                    sig = bytes([27 + i + 4]) + binascii.unhexlify(reply['sign'][0]['sig'])
+                    try:
+                        addr = public_key_to_p2pkh(binascii.unhexlify(reply['sign'][0]['pubkey']))
+                        if verify_message(addr, sig, message):
+                            break
+                    except Exception:
+                        continue
+                else:
+                    raise Exception("Could not sign message")
+
+
         except BaseException as e:
             self.give_error(e)
         return sig
@@ -361,68 +474,84 @@ class DigitalBitbox_KeyStore(Hardware_KeyStore):
         try:
             p2shTransaction = False
             derivations = self.get_tx_derivations(tx)
+            inputhasharray = []
             hasharray = []
             pubkeyarray = []
-            
+
             # Build hasharray from inputs
             for i, txin in enumerate(tx.inputs()):
                 if txin['type'] == 'coinbase':
                     self.give_error("Coinbase not supported") # should never happen
-                
+
                 if txin['type'] in ['p2sh']:
                     p2shTransaction = True
-                
+
                 for x_pubkey in txin['x_pubkeys']:
                     if x_pubkey in derivations:
                         index = derivations.get(x_pubkey)
                         inputPath = "%s/%d/%d" % (self.get_derivation(), index[0], index[1])
-                        inputHash = Hash(tx.serialize_preimage(i).decode('hex')).encode('hex')
-                        hasharray_i = {'hash': inputHash, 'keypath': inputPath}
+                        inputHash = Hash(binascii.unhexlify(tx.serialize_preimage(i)))
+                        hasharray_i = {'hash': to_hexstr(inputHash), 'keypath': inputPath}
                         hasharray.append(hasharray_i)
+                        inputhasharray.append(inputHash)
                         break
                 else:
                     self.give_error("No matching x_key for sign_transaction") # should never happen
-           
+
             # Sanity check
             if p2shTransaction:
                 for txinput in tx.inputs():
                     if txinput['type'] != 'p2sh':
                         self.give_error("P2SH / regular input mixed in same transaction not supported") # should never happen
-            
-            # Build pubkeyarray from outputs (unused because echo for smart verification not implemented)
-            if not p2shTransaction:
-                for _type, address, amount in tx.outputs():
-                    assert _type == TYPE_ADDRESS
-                    info = tx.output_info.get(address)
-                    if info is not None:
-                        index, xpubs, m = info
-                        changePath = self.get_derivation() + "/%d/%d" % index
-                        changePubkey = self.derive_pubkey(index[0], index[1])
-                        pubkeyarray_i = {'pubkey': changePubkey, 'keypath': changePath}
-                        pubkeyarray.append(pubkeyarray_i)
-            
+
+            # Build pubkeyarray from outputs
+            for _type, address, amount in tx.outputs():
+                assert _type == TYPE_ADDRESS
+                info = tx.output_info.get(address)
+                if info is not None:
+                    index, xpubs, m = info
+                    changePath = self.get_derivation() + "/%d/%d" % index
+                    changePubkey = self.derive_pubkey(index[0], index[1])
+                    pubkeyarray_i = {'pubkey': changePubkey, 'keypath': changePath}
+                    pubkeyarray.append(pubkeyarray_i)
+
+            # Special serialization of the unsigned transaction for
+            # the mobile verification app.
+            class CustomTXSerialization(Transaction):
+                @classmethod
+                def input_script(self, txin, estimate_size=False):
+                    if txin['type'] == 'p2pkh':
+                        return Transaction.get_preimage_script(txin)
+                    if txin['type'] == 'p2sh':
+                        return '00' + push_script(Transaction.get_preimage_script(txin))
+                    raise Exception("unsupported type %s" % txin['type'])
+            tx_dbb_serialized = CustomTXSerialization(tx.serialize()).serialize()
+
             # Build sign command
             dbb_signatures = []
             steps = math.ceil(1.0 * len(hasharray) / self.maxInputs)
             for step in range(int(steps)):
                 hashes = hasharray[step * self.maxInputs : (step + 1) * self.maxInputs]
-                
-                msg = '{"sign": {"meta":"%s", "data":%s, "checkpub":%s} }' % \
-                       (Hash(tx.serialize()).encode('hex'), json.dumps(hashes), json.dumps(pubkeyarray))
-                
+
+                msg = ('{"sign": {"meta":"%s", "data":%s, "checkpub":%s} }' % \
+                       (to_hexstr(Hash(tx_dbb_serialized)), json.dumps(hashes), json.dumps(pubkeyarray))).encode('utf8')
                 dbb_client = self.plugin.get_client(self)
-                
+
                 if not dbb_client.is_paired():
                     raise Exception("Could not sign transaction.")
-                
+
                 reply = dbb_client.hid_send_encrypt(msg)
-                
                 if 'error' in reply:
                     raise Exception(reply['error']['message'])
-                
+
                 if 'echo' not in reply:
                     raise Exception("Could not sign transaction.")
-                
+
+                # multisig verification not working correctly yet
+                if self.plugin.is_mobile_paired() and not p2shTransaction:
+                    reply['tx'] = tx_dbb_serialized
+                    self.plugin.comserver_post_notification(reply)
+
                 if steps > 1:
                     self.handler.show_message(_("Signing large transaction. Please be patient ...\r\n\r\n" \
                                                 "To continue, touch the Digital Bitbox's blinking light for 3 seconds. " \
@@ -432,51 +561,60 @@ class DigitalBitbox_KeyStore(Hardware_KeyStore):
                     self.handler.show_message(_("Signing transaction ...\r\n\r\n" \
                                                 "To continue, touch the Digital Bitbox's blinking light for 3 seconds.\r\n\r\n" \
                                                 "To cancel, briefly touch the blinking light or wait for the timeout."))
-                
-                reply = dbb_client.hid_send_encrypt(msg) # Send twice, first returns an echo for smart verification (not implemented)
+
+                # Send twice, first returns an echo for smart verification
+                reply = dbb_client.hid_send_encrypt(msg)
                 self.handler.clear_dialog()
-                
+
                 if 'error' in reply:
                     raise Exception(reply['error']['message'])
-                
+
                 if 'sign' not in reply:
                     raise Exception("Could not sign transaction.")
-                
+
                 dbb_signatures.extend(reply['sign'])
-            
+
             # Fill signatures
-            if len(dbb_signatures) <> len(tx.inputs()):
+            if len(dbb_signatures) != len(tx.inputs()):
                 raise Exception("Incorrect number of transactions signed.") # Should never occur
             for i, txin in enumerate(tx.inputs()):
                 num = txin['num_sig']
                 for pubkey in txin['pubkeys']:
-                    signatures = filter(None, txin['signatures'])
+                    signatures = list(filter(None, txin['signatures']))
                     if len(signatures) == num:
                         break # txin is complete
                     ii = txin['pubkeys'].index(pubkey)
                     signed = dbb_signatures[i]
-                    if signed['pubkey'] != pubkey:
+                    if 'recid' in signed:
+                        # firmware > v2.1.1
+                        recid = int(signed['recid'], 16)
+                        s = binascii.unhexlify(signed['sig'])
+                        h = inputhasharray[i]
+                        pk = MyVerifyingKey.from_signature(s, recid, h, curve = SECP256k1)
+                        pk = to_hexstr(point_to_ser(pk.pubkey.point, True))
+                    elif 'pubkey' in signed:
+                        # firmware <= v2.1.1
+                        pk = signed['pubkey']
+                    if pk != pubkey:
                         continue
                     sig_r = int(signed['sig'][:64], 16)
                     sig_s = int(signed['sig'][64:], 16)
                     sig = sigencode_der(sig_r, sig_s, generator_secp256k1.order())
-                    txin['signatures'][ii] = sig.encode('hex')
+                    txin['signatures'][ii] = to_hexstr(sig) + '01'
                     tx._inputs[i] = txin
-
         except BaseException as e:
             self.give_error(e, True)
         else:
             print_error("Transaction is_complete", tx.is_complete())
             tx.raw = tx.serialize()
-      
 
 
 class DigitalBitboxPlugin(HW_PluginBase):
-    
+
     libraries_available = DIGIBOX
     keystore_class = DigitalBitbox_KeyStore
     client = None
-    DEVICE_IDS = [ 
+    DEVICE_IDS = [
                    (0x03eb, 0x2402) # Digital Bitbox
                  ]
 
@@ -485,6 +623,8 @@ class DigitalBitboxPlugin(HW_PluginBase):
         if self.libraries_available:
             self.device_manager().register_devices(self.DEVICE_IDS)
 
+        self.digitalbitbox_config = self.config.get('digitalbitbox', {})
+
 
     def get_dbb_device(self, device):
         dev = hid.device()
@@ -493,28 +633,49 @@ class DigitalBitboxPlugin(HW_PluginBase):
 
 
     def create_client(self, device, handler):
-        self.handler = handler
-        client = self.get_dbb_device(device)
-        if client <> None:
-            client = DigitalBitbox_Client(client)
-        return client
+        if device.interface_number == 0 or device.usage_page == 0xffff:
+            self.handler = handler
+            client = self.get_dbb_device(device)
+            if client is not None:
+                client = DigitalBitbox_Client(self, client)
+            return client
+        else:
+            return None
 
 
-    def setup_device(self, device_info, wizard):        
+    def setup_device(self, device_info, wizard):
         devmgr = self.device_manager()
         device_id = device_info.device.id_
         client = devmgr.client_by_id(device_id)
         client.handler = self.create_handler(wizard)
         client.setupRunning = True
-        client.get_xpub("m/44'/0'")
+        client.get_xpub("m/44'/0'", 'standard')
 
 
-    def get_xpub(self, device_id, derivation, wizard):
+    def is_mobile_paired(self):
+        return 'encryptionprivkey' in self.digitalbitbox_config
+
+
+    def comserver_post_notification(self, payload):
+        assert self.is_mobile_paired(), "unexpected mobile pairing error"
+        url = 'https://digitalbitbox.com/smartverification/index.php'
+        key_s = base64.b64decode(self.digitalbitbox_config['encryptionprivkey'])
+        args = 'c=data&s=0&dt=0&uuid=%s&pl=%s' % (
+            self.digitalbitbox_config['comserverchannelid'],
+            EncodeAES(key_s, json.dumps(payload).encode('ascii')).decode('ascii'),
+        )
+        try:
+            requests.post(url, args)
+        except Exception as e:
+            self.handler.show_error(str(e))
+
+
+    def get_xpub(self, device_id, derivation, xtype, wizard):
         devmgr = self.device_manager()
         client = devmgr.client_by_id(device_id)
         client.handler = self.create_handler(wizard)
         client.check_device_dialog()
-        xpub = client.get_xpub(derivation)
+        xpub = client.get_xpub(derivation, xtype)
         return xpub
 
 
@@ -522,8 +683,7 @@ class DigitalBitboxPlugin(HW_PluginBase):
         devmgr = self.device_manager()
         handler = keystore.handler
         with devmgr.hid_lock:
-            client = devmgr.client_for_keystore(self, handler, keystore, force_pair)        
-        if client <> None:
+            client = devmgr.client_for_keystore(self, handler, keystore, force_pair)
+        if client is not None:
             client.check_device_dialog()
         return client
-
